@@ -63,7 +63,7 @@ static void srvc_id_to_hal(struct hal_gatt_srvc_id *to, btgatt_srvc_id_t *from)
 
 /* Client Event Handlers */
 
-static void handle_register_client(void *buf, uint16_t len)
+static void handle_register_client(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_register_client *ev = buf;
 
@@ -72,7 +72,7 @@ static void handle_register_client(void *buf, uint16_t len)
 						(bt_uuid_t *) ev->app_uuid);
 }
 
-static void handle_scan_result(void *buf, uint16_t len)
+static void handle_scan_result(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_scan_result *ev = buf;
 	uint8_t ad[62];
@@ -91,7 +91,7 @@ static void handle_scan_result(void *buf, uint16_t len)
 									ad);
 }
 
-static void handle_connect(void *buf, uint16_t len)
+static void handle_connect(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_connect *ev = buf;
 
@@ -100,7 +100,7 @@ static void handle_connect(void *buf, uint16_t len)
 						(bt_bdaddr_t *) ev->bda);
 }
 
-static void handle_disconnect(void *buf, uint16_t len)
+static void handle_disconnect(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_disconnect *ev = buf;
 
@@ -109,7 +109,7 @@ static void handle_disconnect(void *buf, uint16_t len)
 						(bt_bdaddr_t *) ev->bda);
 }
 
-static void handle_search_complete(void *buf, uint16_t len)
+static void handle_search_complete(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_search_complete *ev = buf;
 
@@ -117,7 +117,7 @@ static void handle_search_complete(void *buf, uint16_t len)
 		cbs->client->search_complete_cb(ev->conn_id, ev->status);
 }
 
-static void handle_search_result(void *buf, uint16_t len)
+static void handle_search_result(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_search_result *ev = buf;
 	btgatt_srvc_id_t srvc_id;
@@ -128,7 +128,7 @@ static void handle_search_result(void *buf, uint16_t len)
 		cbs->client->search_result_cb(ev->conn_id, &srvc_id);
 }
 
-static void handle_get_characteristic(void *buf, uint16_t len)
+static void handle_get_characteristic(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_get_characteristic *ev = buf;
 	btgatt_gatt_id_t char_id;
@@ -143,7 +143,7 @@ static void handle_get_characteristic(void *buf, uint16_t len)
 							ev->char_prop);
 }
 
-static void handle_get_descriptor(void *buf, uint16_t len)
+static void handle_get_descriptor(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_get_descriptor *ev = buf;
 	btgatt_gatt_id_t descr_id;
@@ -159,7 +159,7 @@ static void handle_get_descriptor(void *buf, uint16_t len)
 						&srvc_id, &char_id, &descr_id);
 }
 
-static void handle_get_included_service(void *buf, uint16_t len)
+static void handle_get_included_service(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_get_inc_service *ev = buf;
 	btgatt_srvc_id_t srvc_id;
@@ -174,7 +174,7 @@ static void handle_get_included_service(void *buf, uint16_t len)
 								&incl_srvc_id);
 }
 
-static void handle_register_for_notification(void *buf, uint16_t len)
+static void handle_register_for_notification(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_reg_for_notif *ev = buf;
 	btgatt_gatt_id_t char_id;
@@ -191,7 +191,7 @@ static void handle_register_for_notification(void *buf, uint16_t len)
 								&char_id);
 }
 
-static void handle_notify(void *buf, uint16_t len)
+static void handle_notify(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_notify *ev = buf;
 	btgatt_notify_params_t params;
@@ -215,7 +215,7 @@ static void handle_notify(void *buf, uint16_t len)
 		cbs->client->notify_cb(ev->conn_id, &params);
 }
 
-static void handle_read_characteristic(void *buf, uint16_t len)
+static void handle_read_characteristic(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_read_characteristic *ev = buf;
 	btgatt_read_params_t params;
@@ -242,7 +242,7 @@ static void handle_read_characteristic(void *buf, uint16_t len)
 								&params);
 }
 
-static void handle_write_characteristic(void *buf, uint16_t len)
+static void handle_write_characteristic(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_write_characteristic *ev = buf;
 	btgatt_write_params_t params;
@@ -260,7 +260,7 @@ static void handle_write_characteristic(void *buf, uint16_t len)
 								&params);
 }
 
-static void handle_read_descriptor(void *buf, uint16_t len)
+static void handle_read_descriptor(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_read_descriptor *ev = buf;
 	btgatt_read_params_t params;
@@ -287,7 +287,7 @@ static void handle_read_descriptor(void *buf, uint16_t len)
 								&params);
 }
 
-static void handle_write_descriptor(void *buf, uint16_t len)
+static void handle_write_descriptor(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_write_descriptor *ev = buf;
 	btgatt_write_params_t params;
@@ -305,7 +305,7 @@ static void handle_write_descriptor(void *buf, uint16_t len)
 								&params);
 }
 
-static void handle_execute_write(void *buf, uint16_t len)
+static void handle_execute_write(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_exec_write *ev = buf;
 
@@ -313,7 +313,7 @@ static void handle_execute_write(void *buf, uint16_t len)
 		cbs->client->execute_write_cb(ev->conn_id, ev->status);
 }
 
-static void handle_read_remote_rssi(void *buf, uint16_t len)
+static void handle_read_remote_rssi(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_read_remote_rssi *ev = buf;
 
@@ -323,7 +323,7 @@ static void handle_read_remote_rssi(void *buf, uint16_t len)
 						ev->rssi, ev->status);
 }
 
-static void handle_listen(void *buf, uint16_t len)
+static void handle_listen(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_client_listen *ev = buf;
 
@@ -333,7 +333,7 @@ static void handle_listen(void *buf, uint16_t len)
 
 /* Server Event Handlers */
 
-static void handle_register_server(void *buf, uint16_t len)
+static void handle_register_server(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_server_register *ev = buf;
 
@@ -342,7 +342,7 @@ static void handle_register_server(void *buf, uint16_t len)
 						(bt_uuid_t *) &ev->uuid);
 }
 
-static void handle_connection(void *buf, uint16_t len)
+static void handle_connection(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_server_connection *ev = buf;
 
@@ -352,7 +352,7 @@ static void handle_connection(void *buf, uint16_t len)
 						(bt_bdaddr_t *) &ev->bdaddr);
 }
 
-static void handle_service_added(void *buf, uint16_t len)
+static void handle_service_added(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_server_service_added *ev = buf;
 	btgatt_srvc_id_t srvc_id;
@@ -364,7 +364,7 @@ static void handle_service_added(void *buf, uint16_t len)
 						&srvc_id, ev->srvc_handle);
 }
 
-static void handle_included_service_added(void *buf, uint16_t len)
+static void handle_included_service_added(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_server_inc_srvc_added *ev = buf;
 
@@ -375,7 +375,7 @@ static void handle_included_service_added(void *buf, uint16_t len)
 							ev->incl_srvc_handle);
 }
 
-static void handle_characteristic_added(void *buf, uint16_t len)
+static void handle_characteristic_added(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_server_characteristic_added *ev = buf;
 
@@ -386,7 +386,7 @@ static void handle_characteristic_added(void *buf, uint16_t len)
 							ev->char_handle);
 }
 
-static void handle_descriptor_added(void *buf, uint16_t len)
+static void handle_descriptor_added(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_server_descriptor_added *ev = buf;
 
@@ -397,7 +397,7 @@ static void handle_descriptor_added(void *buf, uint16_t len)
 							ev->descr_handle);
 }
 
-static void handle_service_started(void *buf, uint16_t len)
+static void handle_service_started(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_server_service_started *ev = buf;
 
@@ -406,7 +406,7 @@ static void handle_service_started(void *buf, uint16_t len)
 							ev->srvc_handle);
 }
 
-static void handle_service_stopped(void *buf, uint16_t len)
+static void handle_service_stopped(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_server_service_stopped *ev = buf;
 
@@ -415,7 +415,7 @@ static void handle_service_stopped(void *buf, uint16_t len)
 							ev->srvc_handle);
 }
 
-static void handle_service_deleted(void *buf, uint16_t len)
+static void handle_service_deleted(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_server_service_deleted *ev = buf;
 
@@ -424,7 +424,7 @@ static void handle_service_deleted(void *buf, uint16_t len)
 							ev->srvc_handle);
 }
 
-static void handle_request_read(void *buf, uint16_t len)
+static void handle_request_read(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_server_request_read *ev = buf;
 
@@ -435,7 +435,7 @@ static void handle_request_read(void *buf, uint16_t len)
 						ev->is_long);
 }
 
-static void handle_request_write(void *buf, uint16_t len)
+static void handle_request_write(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_server_request_write *ev = buf;
 
@@ -452,7 +452,7 @@ static void handle_request_write(void *buf, uint16_t len)
 						ev->is_prep, ev->value);
 }
 
-static void handle_request_exec_write(void *buf, uint16_t len)
+static void handle_request_exec_write(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_server_request_exec_write *ev = buf;
 
@@ -462,7 +462,7 @@ static void handle_request_exec_write(void *buf, uint16_t len)
 						ev->exec_write);
 }
 
-static void handle_response_confirmation(void *buf, uint16_t len)
+static void handle_response_confirmation(void *buf, uint16_t len, int fd)
 {
 	struct hal_ev_gatt_server_rsp_confirmation *ev = buf;
 
@@ -573,10 +573,13 @@ static bt_status_t register_client(bt_uuid_t *uuid)
 {
 	struct hal_cmd_gatt_client_register cmd;
 
+	if (!interface_ready())
+		return BT_STATUS_NOT_READY;
+
 	memcpy(cmd.uuid, uuid, sizeof(*uuid));
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT, HAL_OP_GATT_CLIENT_REGISTER,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t unregister_client(int client_if)
@@ -589,7 +592,7 @@ static bt_status_t unregister_client(int client_if)
 	cmd.client_if = client_if;
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT, HAL_OP_GATT_CLIENT_UNREGISTER,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t scan(int client_if, bool start)
@@ -603,7 +606,7 @@ static bt_status_t scan(int client_if, bool start)
 	cmd.start = start;
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT, HAL_OP_GATT_CLIENT_SCAN,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t connect(int client_if, const bt_bdaddr_t *bd_addr,
@@ -620,7 +623,7 @@ static bt_status_t connect(int client_if, const bt_bdaddr_t *bd_addr,
 	memcpy(cmd.bdaddr, bd_addr, sizeof(*bd_addr));
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT, HAL_OP_GATT_CLIENT_CONNECT,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t disconnect(int client_if, const bt_bdaddr_t *bd_addr,
@@ -637,7 +640,7 @@ static bt_status_t disconnect(int client_if, const bt_bdaddr_t *bd_addr,
 	memcpy(cmd.bdaddr, bd_addr, sizeof(*bd_addr));
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT, HAL_OP_GATT_CLIENT_DISCONNECT,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t listen(int client_if, bool start)
@@ -651,7 +654,7 @@ static bt_status_t listen(int client_if, bool start)
 	cmd.start = start;
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT, HAL_OP_GATT_CLIENT_LISTEN,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t refresh(int client_if, const bt_bdaddr_t *bd_addr)
@@ -666,7 +669,7 @@ static bt_status_t refresh(int client_if, const bt_bdaddr_t *bd_addr)
 	memcpy(cmd.bdaddr, bd_addr, sizeof(*bd_addr));
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT, HAL_OP_GATT_CLIENT_REFRESH,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t search_service(int conn_id, bt_uuid_t *filter_uuid)
@@ -690,7 +693,7 @@ static bt_status_t search_service(int conn_id, bt_uuid_t *filter_uuid)
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 					HAL_OP_GATT_CLIENT_SEARCH_SERVICE,
-					len, cmd, 0, NULL, NULL);
+					len, cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t get_included_service(int conn_id, btgatt_srvc_id_t *srvc_id,
@@ -716,7 +719,7 @@ static bt_status_t get_included_service(int conn_id, btgatt_srvc_id_t *srvc_id,
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 					HAL_OP_GATT_CLIENT_GET_INCLUDED_SERVICE,
-					len, cmd, 0, NULL, NULL);
+					len, cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t get_characteristic(int conn_id, btgatt_srvc_id_t *srvc_id,
@@ -742,7 +745,7 @@ static bt_status_t get_characteristic(int conn_id, btgatt_srvc_id_t *srvc_id,
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 					HAL_OP_GATT_CLIENT_GET_CHARACTERISTIC,
-					len, cmd, 0, NULL, NULL);
+					len, cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t get_descriptor(int conn_id, btgatt_srvc_id_t *srvc_id,
@@ -770,7 +773,7 @@ static bt_status_t get_descriptor(int conn_id, btgatt_srvc_id_t *srvc_id,
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 					HAL_OP_GATT_CLIENT_GET_DESCRIPTOR,
-					len, cmd, 0 , NULL, NULL);
+					len, cmd, NULL , NULL, NULL);
 }
 
 static bt_status_t read_characteristic(int conn_id, btgatt_srvc_id_t *srvc_id,
@@ -790,7 +793,7 @@ static bt_status_t read_characteristic(int conn_id, btgatt_srvc_id_t *srvc_id,
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 					HAL_OP_GATT_CLIENT_READ_CHARACTERISTIC,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t write_characteristic(int conn_id, btgatt_srvc_id_t *srvc_id,
@@ -817,7 +820,7 @@ static bt_status_t write_characteristic(int conn_id, btgatt_srvc_id_t *srvc_id,
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 					HAL_OP_GATT_CLIENT_WRITE_CHARACTERISTIC,
-					cmd_len, cmd, 0, NULL, NULL);
+					cmd_len, cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t read_descriptor(int conn_id, btgatt_srvc_id_t *srvc_id,
@@ -839,7 +842,7 @@ static bt_status_t read_descriptor(int conn_id, btgatt_srvc_id_t *srvc_id,
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 					HAL_OP_GATT_CLIENT_READ_DESCRIPTOR,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t write_descriptor(int conn_id, btgatt_srvc_id_t *srvc_id,
@@ -868,7 +871,7 @@ static bt_status_t write_descriptor(int conn_id, btgatt_srvc_id_t *srvc_id,
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 					HAL_OP_GATT_CLIENT_WRITE_DESCRIPTOR,
-					cmd_len, cmd, 0, NULL, NULL);
+					cmd_len, cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t execute_write(int conn_id, int execute)
@@ -883,7 +886,7 @@ static bt_status_t execute_write(int conn_id, int execute)
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 					HAL_OP_GATT_CLIENT_EXECUTE_WRITE,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t register_for_notification(int client_if,
@@ -905,7 +908,7 @@ static bt_status_t register_for_notification(int client_if,
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 				HAL_OP_GATT_CLIENT_REGISTER_FOR_NOTIFICATION,
-				sizeof(cmd), &cmd, 0, NULL, NULL);
+				sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t deregister_for_notification(int client_if,
@@ -927,7 +930,7 @@ static bt_status_t deregister_for_notification(int client_if,
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 				HAL_OP_GATT_CLIENT_DEREGISTER_FOR_NOTIFICATION,
-				sizeof(cmd), &cmd, 0, NULL, NULL);
+				sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t read_remote_rssi(int client_if, const bt_bdaddr_t *bd_addr)
@@ -943,7 +946,7 @@ static bt_status_t read_remote_rssi(int client_if, const bt_bdaddr_t *bd_addr)
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 					HAL_OP_GATT_CLIENT_READ_REMOTE_RSSI,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static int get_device_type(const bt_bdaddr_t *bd_addr)
@@ -968,19 +971,28 @@ static int get_device_type(const bt_bdaddr_t *bd_addr)
 	return dev_type;
 }
 
-static bt_status_t set_adv_data(int server_if, bool set_scan_rsp,
+static bt_status_t set_adv_data_real(int server_if, bool set_scan_rsp,
 				bool include_name, bool include_txpower,
 				int min_interval, int max_interval,
 				int appearance, uint16_t manufacturer_len,
-				char *manufacturer_data)
+				char *manufacturer_data,
+				uint16_t service_data_len, char *service_data,
+				uint16_t service_uuid_len, char *service_uuid)
 {
 	char buf[IPC_MTU];
 	struct hal_cmd_gatt_client_set_adv_data *cmd = (void *) buf;
-	size_t cmd_len = sizeof(*cmd) + manufacturer_len;
+	size_t cmd_len;
+	uint8_t *data;
 
 	if (!interface_ready())
 		return BT_STATUS_NOT_READY;
 
+	cmd_len = sizeof(*cmd) + manufacturer_len + service_data_len +
+							service_uuid_len;
+
+	if (cmd_len > IPC_MTU)
+		return BT_STATUS_FAIL;
+
 	cmd->server_if = server_if;
 	cmd->set_scan_rsp = set_scan_rsp;
 	cmd->include_name = include_name;
@@ -989,13 +1001,63 @@ static bt_status_t set_adv_data(int server_if, bool set_scan_rsp,
 	cmd->max_interval = max_interval;
 	cmd->appearance = appearance;
 	cmd->manufacturer_len = manufacturer_len;
+	cmd->service_data_len = service_data_len;
+	cmd->service_uuid_len = service_uuid_len;
 
-	memcpy(cmd->manufacturer_data, manufacturer_data, manufacturer_len);
+	data = cmd->data;
+
+	if (manufacturer_data && manufacturer_len) {
+		memcpy(data, manufacturer_data, manufacturer_len);
+		data += manufacturer_len;
+	}
+
+	if (service_data && service_data_len) {
+		memcpy(data, service_data, service_data_len);
+		data += service_data_len;
+	}
+
+	if (service_uuid && service_uuid_len)
+		memcpy(data, service_uuid, service_uuid_len);
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT, HAL_OP_GATT_CLIENT_SET_ADV_DATA,
-						cmd_len, cmd, 0, NULL, NULL);
+						cmd_len, cmd, NULL, NULL, NULL);
 }
 
+/*
+ * This is temporary solution and support for older Android versions might
+ * be removed at any time.
+ */
+#if ANDROID_VERSION < PLATFORM_VER(4,4,3)
+static bt_status_t set_adv_data(int server_if, bool set_scan_rsp,
+				bool include_name, bool include_txpower,
+				int min_interval, int max_interval,
+				int appearance, uint16_t manufacturer_len,
+				char *manufacturer_data)
+{
+	return set_adv_data_real(server_if, set_scan_rsp, include_name,
+					include_txpower, min_interval,
+					max_interval, appearance,
+					manufacturer_len, manufacturer_data,
+					0, NULL, 0, NULL);
+}
+#else
+static bt_status_t set_adv_data(int server_if, bool set_scan_rsp,
+				bool include_name, bool include_txpower,
+				int min_interval, int max_interval,
+				int appearance, uint16_t manufacturer_len,
+				char *manufacturer_data,
+				uint16_t service_data_len, char *service_data,
+				uint16_t service_uuid_len, char *service_uuid)
+{
+	return set_adv_data_real(server_if, set_scan_rsp, include_name,
+					include_txpower, min_interval,
+					max_interval, appearance,
+					manufacturer_len, manufacturer_data,
+					service_data_len, service_data,
+					service_uuid_len, service_uuid);
+}
+#endif
+
 static bt_status_t test_command(int command, btgatt_test_params_t *params)
 {
 	struct hal_cmd_gatt_client_test_command cmd;
@@ -1015,7 +1077,7 @@ static bt_status_t test_command(int command, btgatt_test_params_t *params)
 	cmd.u5 = params->u5;
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT, HAL_OP_GATT_CLIENT_TEST_COMMAND,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 /* Server API */
@@ -1030,7 +1092,7 @@ static bt_status_t register_server(bt_uuid_t *uuid)
 	memcpy(cmd.uuid, uuid, sizeof(*uuid));
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT, HAL_OP_GATT_SERVER_REGISTER,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t unregister_server(int server_if)
@@ -1043,7 +1105,7 @@ static bt_status_t unregister_server(int server_if)
 	cmd.server_if = server_if;
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT, HAL_OP_GATT_SERVER_UNREGISTER,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t server_connect(int server_if, const bt_bdaddr_t *bd_addr,
@@ -1060,7 +1122,7 @@ static bt_status_t server_connect(int server_if, const bt_bdaddr_t *bd_addr,
 	memcpy(cmd.bdaddr, bd_addr, sizeof(*bd_addr));
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT, HAL_OP_GATT_SERVER_CONNECT,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t server_disconnect(int server_if, const bt_bdaddr_t *bd_addr,
@@ -1077,7 +1139,7 @@ static bt_status_t server_disconnect(int server_if, const bt_bdaddr_t *bd_addr,
 	memcpy(cmd.bdaddr, bd_addr, sizeof(*bd_addr));
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT, HAL_OP_GATT_SERVER_DISCONNECT,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t add_service(int server_if, btgatt_srvc_id_t *srvc_id,
@@ -1094,7 +1156,7 @@ static bt_status_t add_service(int server_if, btgatt_srvc_id_t *srvc_id,
 	srvc_id_to_hal(&cmd.srvc_id, srvc_id);
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT, HAL_OP_GATT_SERVER_ADD_SERVICE,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t add_included_service(int server_if, int service_handle,
@@ -1111,7 +1173,7 @@ static bt_status_t add_included_service(int server_if, int service_handle,
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 					HAL_OP_GATT_SERVER_ADD_INC_SERVICE,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t add_characteristic(int server_if, int service_handle,
@@ -1132,7 +1194,7 @@ static bt_status_t add_characteristic(int server_if, int service_handle,
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 					HAL_OP_GATT_SERVER_ADD_CHARACTERISTIC,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t add_descriptor(int server_if, int service_handle,
@@ -1151,7 +1213,7 @@ static bt_status_t add_descriptor(int server_if, int service_handle,
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 					HAL_OP_GATT_SERVER_ADD_DESCRIPTOR,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t start_service(int server_if, int service_handle,
@@ -1168,7 +1230,7 @@ static bt_status_t start_service(int server_if, int service_handle,
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 					HAL_OP_GATT_SERVER_START_SERVICE,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t stop_service(int server_if, int service_handle)
@@ -1182,7 +1244,7 @@ static bt_status_t stop_service(int server_if, int service_handle)
 	cmd.service_handle = service_handle;
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT, HAL_OP_GATT_SERVER_STOP_SERVICE,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t delete_service(int server_if, int service_handle)
@@ -1197,7 +1259,7 @@ static bt_status_t delete_service(int server_if, int service_handle)
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 					HAL_OP_GATT_SERVER_DELETE_SERVICE,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t send_indication(int server_if, int attribute_handle,
@@ -1221,7 +1283,7 @@ static bt_status_t send_indication(int server_if, int attribute_handle,
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 					HAL_OP_GATT_SERVER_SEND_INDICATION,
-					cmd_len, cmd, 0, NULL, NULL);
+					cmd_len, cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t send_response(int conn_id, int trans_id, int status,
@@ -1248,7 +1310,7 @@ static bt_status_t send_response(int conn_id, int trans_id, int status,
 
 	return hal_ipc_cmd(HAL_SERVICE_ID_GATT,
 					HAL_OP_GATT_SERVER_SEND_RESPONSE,
-					cmd_len, cmd, 0, NULL, NULL);
+					cmd_len, cmd, NULL, NULL, NULL);
 }
 
 static bt_status_t init(const btgatt_callbacks_t *callbacks)
@@ -1270,7 +1332,7 @@ static bt_status_t init(const btgatt_callbacks_t *callbacks)
 	cmd.mode = HAL_MODE_DEFAULT;
 
 	ret = hal_ipc_cmd(HAL_SERVICE_ID_CORE, HAL_OP_REGISTER_MODULE,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 
 	if (ret != BT_STATUS_SUCCESS) {
 		cbs = NULL;
@@ -1294,7 +1356,7 @@ static void cleanup(void)
 	cmd.service_id = HAL_SERVICE_ID_GATT;
 
 	hal_ipc_cmd(HAL_SERVICE_ID_CORE, HAL_OP_UNREGISTER_MODULE,
-					sizeof(cmd), &cmd, 0, NULL, NULL);
+					sizeof(cmd), &cmd, NULL, NULL, NULL);
 
 	hal_ipc_unregister(HAL_SERVICE_ID_GATT);
 }
